const { _TempContext } = require('./temp-context')

/**
 * A test context that creates and destroys a temp directory. By default, the temp directory will be `test/temp` relative to the current working directory, but it can be changed by extending the class and setting its `TEMP` property in the constructor.
 */
class TempContext extends _TempContext {
  /**
   * The constructor method will be called by a context-testing framework automatically.
   */
  constructor() {
    super()
  }
  /**
   * This method should be called in the constructor by classes that extend the `TempContext` to use the temp directory of the os system.
   * @param {string} name The name of the directory inside of the temp dir.
   */
  _useOSTemp(name) {
    return super._useOSTemp(name)
  }
  /**
   * Read a file from the filesystem.
   * @param {string} path Path of the file to read.
   * @return {!Promise<string>}
   */
  async readGlobal(path) {
    return super.readGlobal(path)
  }
  /**
   * Check if the path exists on the filesystem.
   * @param {string} path The path to check.
   * @return {!Promise<boolean>}
   */
  async existsGlobal(path) {
    return super.existsGlobal(path)
  }
  /**
   * Check if the path exists in the temp directory.
   * @param {string} path The relative path inside of the temp dir to check.
   * @return {!Promise<boolean>}
   */
  async exists(path) {
    return super.exists(path)
  }
  /**
   * Read a file inside of the temp directory.
   * @param {string} path The path to the file in the temp directory.
   * @return {!Promise<string>}
   */
  async read(path) {
    return super.read(path)
  }
  /**
   * Write a file in the temp directory.
   * @param {string} path The path to the file within the temp directory.
   * @param {(string|!Buffer)} data The data to write.
   * @return {!Promise<string>}
   */
  async write(path, data) {
    return super.write(path, data)
  }
  /**
   * Get a path to a file inside of the temp directory.
   * @param {string} path The relative path to the file inside of the temp dir.
   * @return {string}
   */
  resolve(path) {
    return super.resolve(path)
  }
  /**
   * Remove a file or folder inside of the temp directory.
   * @param {string} path The path of the file or folder to remove.
   * @return {!Promise}
   */
  async rm(path) {
    return super.rm(path)
  }
  /**
   * Clone a file or directory. This works like `cloneGlobal`, such that the path won't be resolved automatically.
   * @param {string} path The path to the file or directory to clone.
   * @param {string} to The path to the directory to contain the file or directory being cloned (not the path to the cloned entity).
   * @return {!Promise}
   */
  async clone(path, to) {
    return super.clone(path, to)
  }
  /**
   * Adds a file or directory to the temp directory and returns its new path.
   * @param {string} target The path to the file or directory to add to the temp directory.
   * @return {!Promise<string>}
   */
  async add(target) {
    return super.add(target)
  }
  /**
   * Capture the contents of the temp directory as a string (or partial contents if the inner path is given).
   * @param {(string|!_contextTesting.SnapshotOptions)=} [options] When a string is passed, indicates the inner path (left for compatibility with previous API). Otherwise, these are additional options for the snapshot, where inner path can also be specified.
   * @return {!Promise<string>}
   */
  async snapshot(options) {
    return super.snapshot(options)
  }
  /**
   * Called automatically by a context-testing framework to remove the temp path.
   * @return {!Promise}
   */
  async _destroy() {
    return super._destroy()
  }
  /**
   * Called automatically by a context-testing framework to create an empty temp dir.
   * @return {!Promise}
   */
  async _init() {
    return super._init()
  }
}

module.exports = TempContext

/* typal types/index.xml ignore:_contextTesting.TempContext namespace */
/**
 * @typedef {_contextTesting.SnapshotOptions} SnapshotOptions `＠record`
 * @typedef {Object} _contextTesting.SnapshotOptions `＠record`
 * @prop {boolean} [posix=false] Standardise the path into POSIX (`dir/file.txt`) &mdash; this is useful when tests are run on Windows too. Default `false`.
 * @prop {string} [innerPath="."] The path inside of the temp dir to snapshot. Default `.`.
 */
